"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0d4ec0f15c61

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsCardSecurityResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The level of security applied during card processing."""

    NORMAL = "normal"
    THREEDSECURE = "3dsecure"
