"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f59e911c2d91

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsCardFundingResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The card type."""

    DEBIT = "debit"
    CREDIT = "credit"
    PREPAID = "prepaid"
    DEFERRED_DEBIT = "deferred-debit"
