"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2844902d65e4

from __future__ import annotations
from enum import Enum


class BalanceTransferPartyType(str, Enum):
    r"""Defines the type of the party. At the moment, only `organization` is supported."""

    ORGANIZATION = "organization"
