"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 3d59439f65ce

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsCardAuditionResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The card's target audience, if known."""

    CONSUMER = "consumer"
    BUSINESS = "business"
