"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b05cb4aa8e7f

from __future__ import annotations
from enum import Enum
from mollie import utils


class SalesInvoiceDiscountTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The type of discount."""

    AMOUNT = "amount"
    PERCENTAGE = "percentage"
