"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 896c052f4387

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceTransferDestinationType(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The default destination of automatic scheduled transfers. Currently only `bank-account` is supported.

    * `bank-account` — Transfer the balance amount to an external bank account
    """

    BANK_ACCOUNT = "bank-account"
