"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: bab7a395c5ff

from __future__ import annotations
from enum import Enum


class MethodResourceParameter(str, Enum):
    PAYMENTS = "payments"
    ORDERS = "orders"
