"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ccfe5d251516

from __future__ import annotations
from enum import Enum


class MethodIncludeWalletsParameter(str, Enum):
    APPLEPAY = "applepay"
