"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 09931fdd8e1d

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsReceiptCardReadMethodResponse(
    str, Enum, metaclass=utils.OpenEnumMeta
):
    r"""The method by which the card was read by the terminal."""

    CHIP = "chip"
    MAGNETIC_STRIPE = "magnetic-stripe"
    NEAR_FIELD_COMMUNICATION = "near-field-communication"
    CONTACTLESS = "contactless"
    MOTO = "moto"
