"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6f5ad4890326

from __future__ import annotations
from mollie.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AddressTypedDict(TypedDict):
    street_and_number: str
    r"""A street and street number."""
    postal_code: str
    r"""A postal code. This field may be required if the provided country has a postal code system."""
    city: str
    country: str
    r"""A country code in [ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) format."""


class Address(BaseModel):
    street_and_number: Annotated[str, pydantic.Field(alias="streetAndNumber")]
    r"""A street and street number."""

    postal_code: Annotated[str, pydantic.Field(alias="postalCode")]
    r"""A postal code. This field may be required if the provided country has a postal code system."""

    city: str

    country: str
    r"""A country code in [ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) format."""


try:
    Address.model_rebuild()
except NameError:
    pass
