"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 4f253362d58f

from __future__ import annotations
from enum import Enum


class Sorting(str, Enum):
    ASC = "asc"
    DESC = "desc"
