"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0982f5c9d2b2

from __future__ import annotations
from enum import Enum
from mollie import utils


class MethodStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The payment method's activation status for this profile."""

    ACTIVATED = "activated"
    PENDING_BOARDING = "pending-boarding"
    PENDING_REVIEW = "pending-review"
    PENDING_EXTERNAL = "pending-external"
    REJECTED = "rejected"
