"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f29be1cf56ff

from __future__ import annotations
from enum import Enum
from mollie import utils


class TerminalBrand(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The brand of the terminal."""

    PAX = "PAX"
    TAP = "Tap"
