"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 428830cc5b50

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalancePrepaymentPartType(str, Enum, metaclass=utils.OpenEnumMeta):
    FEE = "fee"
    FEE_REIMBURSEMENT = "fee-reimbursement"
    FEE_DISCOUNT = "fee-discount"
    FEE_VAT = "fee-vat"
    FEE_ROUNDING_COMPENSATION = "fee-rounding-compensation"
