"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ed89386c0ab6

from __future__ import annotations
from enum import Enum


class SalesInvoiceVatMode(str, Enum):
    r"""The VAT mode to use for VAT calculation. `exclusive` mode means we will apply the relevant VAT on top of the
    price. `inclusive` means the prices you are providing to us already contain the VAT you want to apply.
    """

    EXCLUSIVE = "exclusive"
    INCLUSIVE = "inclusive"
