"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 4b06082dfd79

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceTransferCategoryResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The type of the transfer. Different fees may apply to different types of transfers."""

    INVOICE_COLLECTION = "invoice_collection"
    PURCHASE = "purchase"
    CHARGEBACK = "chargeback"
    REFUND = "refund"
    SERVICE_PENALTY = "service_penalty"
    DISCOUNT_COMPENSATION = "discount_compensation"
    MANUAL_CORRECTION = "manual_correction"
    OTHER_FEE = "other_fee"
