"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 4e0005a81524

from __future__ import annotations
from enum import Enum
from mollie import utils


class RouteDestinationTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The type of destination. Currently only the destination type `organization` is supported."""

    ORGANIZATION = "organization"
