"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 479c1789cf72

from __future__ import annotations
from enum import Enum


class OnboardingVatRegulation(str, Enum):
    r"""Mollie applies Dutch VAT for merchants based in The Netherlands, British VAT for merchants based in
    The United Kingdom, and shifted VAT for merchants in the European Union.

    The field can be omitted for merchants residing in other countries.
    """

    DUTCH = "dutch"
    BRITISH = "british"
    SHIFTED = "shifted"
