"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 67347c3fd9ae

from __future__ import annotations
from enum import Enum
from mollie import utils


class SalesInvoiceRecipientLocaleResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The locale for the recipient, to be used for translations in PDF generation and payment pages."""

    EN_US = "en_US"
    EN_GB = "en_GB"
    NL_NL = "nl_NL"
    NL_BE = "nl_BE"
    DE_DE = "de_DE"
    DE_AT = "de_AT"
    DE_CH = "de_CH"
    FR_FR = "fr_FR"
    FR_BE = "fr_BE"
