"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 455d66223a80

from __future__ import annotations
from enum import Enum
from mollie import utils


class CapabilityStatusReason(str, Enum, metaclass=utils.OpenEnumMeta):
    REQUIREMENT_PAST_DUE = "requirement-past-due"
    ONBOARDING_INFORMATION_NEEDED = "onboarding-information-needed"
