"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f2eab98f0867

from __future__ import annotations
from enum import Enum
from mollie import utils


class ProfileReviewStatusResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The status of the requested changes."""

    PENDING = "pending"
    REJECTED = "rejected"
