"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0fdad090d20f

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceCardIssuer(str, Enum, metaclass=utils.OpenEnumMeta):
    AMEX = "amex"
    MAESTRO = "maestro"
    CARTE_BANCAIRE = "carte-bancaire"
    OTHER = "other"
