"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: a050e0645735

from __future__ import annotations
from enum import Enum
from mollie import utils


class RefundExternalReferenceTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""Specifies the reference type"""

    ACQUIRER_REFERENCE = "acquirer-reference"
