"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: bb6971304123

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsSellerProtectionResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""Indicates to what extent the payment is eligible for PayPal's Seller Protection. Only available for PayPal
    payments, and if the information is made available by PayPal.
    """

    ELIGIBLE_UPPER = "ELIGIBLE"
    PARTIALLY_ELIGIBLE_UPPER = "PARTIALLY_ELIGIBLE"
    NOT_ELIGIBLE = "NOT_ELIGIBLE"
    # Deprecated value
    ELIGIBLE_MIXED = "Eligible"
    # Deprecated value
    INELIGIBLE = "Ineligible"
    # Deprecated value
    PARTIALLY_ELIGIBLE_INR_ONLY = "Partially Eligible - INR Only"
    # Deprecated value
    PARTIALLY_ELIGIBLE_UNAUTH_ONLY = "Partially Eligible - Unauth Only"
    # Deprecated value
    PARTIALLY_ELIGIBLE_MIXED = "Partially Eligible"
    # Deprecated value
    NONE = "None"
    # Deprecated value
    ACTIVE = "Active"
    # Deprecated value
    FRAUD_CONTROL_UNAUTH_PREMIUM_ELIGIBLE = "Fraud Control - Unauth Premium Eligible"
