"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f74d7238864f

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsWalletResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The wallet used when creating the payment."""

    APPLEPAY = "applepay"
