"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 38f7e9d04e0e

from __future__ import annotations
from enum import Enum
from mollie import utils


class TerminalModel(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The model of the terminal. For example for a PAX A920, this field's value will be `A920`."""

    A35 = "A35"
    A77 = "A77"
    A920 = "A920"
    A920_PRO = "A920Pro"
    IM30 = "IM30"
    TAP = "Tap"
