"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c6c134a2a415

from __future__ import annotations
from enum import Enum
from mollie import utils


class CapabilityRequirementStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The status of the requirement depends on its due date.
    If no due date is given, the status will be `requested`.
    """

    CURRENTLY_DUE = "currently-due"
    PAST_DUE = "past-due"
    REQUESTED = "requested"
