"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6a131ef7b508

from __future__ import annotations
from enum import Enum


class SessionSequenceType(str, Enum):
    ONEOFF = "oneoff"
    FIRST = "first"
