"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 450da8431c7d

from __future__ import annotations
from enum import Enum
from mollie import utils


class WebhookStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The subscription's current status."""

    ENABLED = "enabled"
    BLOCKED = "blocked"
    DISABLED = "disabled"
    DELETED = "deleted"
