"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 50695ce875be

from __future__ import annotations
from enum import Enum
from mollie import utils


class SalesInvoiceRecipientTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The type of recipient, either `consumer` or `business`. This will determine what further fields are
    required on the `recipient` object.
    """

    CONSUMER = "consumer"
    BUSINESS = "business"
