"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2acf39c126b2

from __future__ import annotations
from enum import Enum
from mollie import utils


class SubscriptionMethodResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The payment method used for this subscription. If omitted, any of the customer's valid mandates may be used."""

    CREDITCARD = "creditcard"
    DIRECTDEBIT = "directdebit"
    PAYPAL = "paypal"
