"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: cfc391ebb5f9

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceTransferStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The status of the transfer."""

    CREATED = "created"
    FAILED = "failed"
    SUCCEEDED = "succeeded"
