"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 7da6f4afe6bb

from __future__ import annotations
from enum import Enum
from mollie import utils


class CapabilityStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    UNREQUESTED = "unrequested"
    ENABLED = "enabled"
    DISABLED = "disabled"
    PENDING = "pending"
