"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f9ec93f73c31

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsReceiptCardVerificationMethodResponse(
    str, Enum, metaclass=utils.OpenEnumMeta
):
    r"""The method used to verify the cardholder's identity."""

    NO_CVM_REQUIRED = "no-cvm-required"
    ONLINE_PIN = "online-pin"
    OFFLINE_PIN = "offline-pin"
    CONSUMER_DEVICE = "consumer-device"
    SIGNATURE = "signature"
    SIGNATURE_AND_ONLINE_PIN = "signature-and-online-pin"
    ONLINE_PIN_AND_SIGNATURE = "online-pin-and-signature"
    NONE = "none"
    FAILED = "failed"
