"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f9427570201d

from __future__ import annotations
from mollie.types import BaseModel
from typing_extensions import TypedDict


class SalesInvoiceEmailDetailsTypedDict(TypedDict):
    subject: str
    r"""The subject of the email to be sent."""
    body: str
    r"""The body of the email to be sent. To add newline characters, you can use `\n`."""


class SalesInvoiceEmailDetails(BaseModel):
    subject: str
    r"""The subject of the email to be sent."""

    body: str
    r"""The body of the email to be sent. To add newline characters, you can use `\n`."""
