"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: bc55d2395646

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceTransferPartyTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""Defines the type of the party. At the moment, only `organization` is supported."""

    ORGANIZATION = "organization"
