"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 39ce795e444f

from __future__ import annotations
from enum import Enum


class SalesInvoiceDiscountType(str, Enum):
    r"""The type of discount."""

    AMOUNT = "amount"
    PERCENTAGE = "percentage"
