"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: d323add59ea0

from __future__ import annotations
from enum import Enum


class LineCategories(str, Enum):
    ECO = "eco"
    GIFT = "gift"
    MEAL = "meal"
    SPORT_CULTURE = "sport_culture"
    ADDITIONAL = "additional"
    CONSUME = "consume"
