"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 8e6a4024c7b5

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceTransferStatusReasonResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""A machine-readable code that indicates the reason for the transfer's status."""

    REQUEST_CREATED = "request_created"
    SUCCESS = "success"
    SOURCE_NOT_ALLOWED = "source_not_allowed"
    DESTINATION_NOT_ALLOWED = "destination_not_allowed"
    INSUFFICIENT_FUNDS = "insufficient_funds"
    INVALID_SOURCE_BALANCE = "invalid_source_balance"
    INVALID_DESTINATION_BALANCE = "invalid_destination_balance"
    TRANSFER_REQUEST_EXPIRED = "transfer_request_expired"
    TRANSFER_LIMIT_REACHED = "transfer_limit_reached"
