"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2facf3881ed6

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceCardAudience(str, Enum, metaclass=utils.OpenEnumMeta):
    CORPORATE = "corporate"
    OTHER = "other"
