"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: e2ffaff1a410

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceCardRegion(str, Enum, metaclass=utils.OpenEnumMeta):
    INTRA_EEA = "intra-eea"
    INTRA_EU = "intra-eu"
    DOMESTIC = "domestic"
    OTHER = "other"
