"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ae7f9ad7afd8

from __future__ import annotations
from enum import Enum
from mollie import utils


class SessionSequenceTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    ONEOFF = "oneoff"
    FIRST = "first"
