"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: d5aedbf611fa

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsFeeRegionResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The applicable card fee region."""

    AMERICAN_EXPRESS = "american-express"
    AMEX_INTRA_EEA = "amex-intra-eea"
    CARTE_BANCAIRE = "carte-bancaire"
    INTRA_EU = "intra-eu"
    INTRA_EU_CORPORATE = "intra-eu-corporate"
    DOMESTIC = "domestic"
    MAESTRO = "maestro"
    OTHER = "other"
    INTER = "inter"
    INTRA_EEA = "intra_eea"
