"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f690303e4481

from typing import Any, TYPE_CHECKING

from mollie.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .globals import Globals, GlobalsTypedDict

__all__ = ["Globals", "GlobalsTypedDict"]

_dynamic_imports: dict[str, str] = {
    "Globals": ".globals",
    "GlobalsTypedDict": ".globals",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)
