"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 90306aaaefeb

from __future__ import annotations
from .sales_invoice_discount_type import SalesInvoiceDiscountType
from mollie.types import BaseModel
from typing_extensions import TypedDict


class SalesInvoiceDiscountTypedDict(TypedDict):
    type: SalesInvoiceDiscountType
    r"""The type of discount."""
    value: str
    r"""A string containing an exact monetary amount in the given currency, or the percentage."""


class SalesInvoiceDiscount(BaseModel):
    type: SalesInvoiceDiscountType
    r"""The type of discount."""

    value: str
    r"""A string containing an exact monetary amount in the given currency, or the percentage."""
