"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2c53eca910ba

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceReportGrouping(str, Enum, metaclass=utils.OpenEnumMeta):
    STATUS_BALANCES = "status-balances"
    TRANSACTION_CATEGORIES = "transaction-categories"
