"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 4f87e7da00bc

from __future__ import annotations
from enum import Enum
from mollie import utils


class SalesInvoiceVatSchemeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The VAT scheme to create the invoice for. You must be enrolled with One Stop Shop enabled to use it."""

    STANDARD = "standard"
    ONE_STOP_SHOP = "one-stop-shop"
