"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 171156f348d2

from __future__ import annotations
from .balance_transfer_party_type import BalanceTransferPartyType
from mollie.types import BaseModel
from typing_extensions import TypedDict


class EntityBalanceTransferPartyTypedDict(TypedDict):
    r"""A party involved in the balance transfer, either the sender or the receiver."""

    type: BalanceTransferPartyType
    r"""Defines the type of the party. At the moment, only `organization` is supported."""
    id: str
    description: str
    r"""The transfer description for the transfer party. This is the description that will appear in the financial reports of the party."""


class EntityBalanceTransferParty(BaseModel):
    r"""A party involved in the balance transfer, either the sender or the receiver."""

    type: BalanceTransferPartyType
    r"""Defines the type of the party. At the moment, only `organization` is supported."""

    id: str

    description: str
    r"""The transfer description for the transfer party. This is the description that will appear in the financial reports of the party."""
